@@ -21,7 +21,7 @@
  #include <libdefs.h>
 #endif
 #include <bzero.h>
-#include <xmemory.h>
+#include <swap.h>
 #include <lcrypt.h>
 #include <blowfish.h>
 #include <3-way.h>
@@ -31,17 +31,20 @@
 #include <cast.h>
 #include <tean.h>
 #include <twofish.h>
+#include <xmemory.h>
 
-#define MAX_THREADS 30000 /* After that pointers are being reused */
+#define MAX_THREADS 256 /* After that pointers are being reused */
 
 /* I think now is re-entrant */
 
+static int check_pointer[MAX_THREADS]={0};
+
 #ifdef HAVE_LIBPTHREAD
 # include <pthread.h>
-static int cur_thread=-1;
+static int cur_thread=0;
 pthread_mutex_t counter_cur_thread = PTHREAD_MUTEX_INITIALIZER;
 #else
-static int cur_thread=-1; /* No threads */
+static int cur_thread=0; /* No threads */
 #endif
 
 static struct cast_key* ckey=NULL; /* For cast */
@@ -53,7 +56,6 @@ static char **kn1=NULL; /* For DES */
 static char **kn2=NULL;
 static char **kn3=NULL; /* For tripleDES */
 static word8** saferkey=NULL; /* For safer64 and 128 */
-static int finished_threads=0;
 
 /* For CBC mode */
 static word32 **previous_ciphertext;
@@ -142,69 +144,70 @@ int get_key_size(int algorithm) /* In bytes */
 }
 
 
-int init_mcrypt(const int algorithm, void * key, int lenofkey)
+int init_mcrypt(const int algorithm, int thread, void * key, int lenofkey)
 {
 
 char *keyword1, *keyword2, *keyword3; /* For 3DES and safer*/
 unsigned char *skey=NULL;
 word8 twokey[16]; /* 128 bits */
 
-  saferkey=mxrealloc(saferkey, (cur_thread+1)*sizeof(char*));
-  kn1=mxrealloc(kn1, (cur_thread+1)*sizeof(char*));
-  kn2=mxrealloc(kn2, (cur_thread+1)*sizeof(char*));
-  kn3=mxrealloc(kn3, (cur_thread+1)*sizeof(char*));
-  c=mxrealloc(c, (cur_thread+1) * sizeof(blf_ctx));
-  CT=mxrealloc(CT, (cur_thread+1) * sizeof(TWOFISH_context));
-  ckey=mxrealloc(ckey, (cur_thread+1) * sizeof(struct cast_key));
-  keyword_given=mxrealloc(keyword_given, (cur_thread+1)*sizeof(char*));
-  algorithm_given=mxrealloc(algorithm_given, (cur_thread+1)*sizeof(int));
-
-
-  if (get_key_size(algorithm) < lenofkey) lenofkey=get_key_size(algorithm);
+  saferkey=mxrealloc(saferkey, (thread+1)*sizeof(char*));
+  kn1=mxrealloc(kn1, (thread+1)*sizeof(char*));
+  kn2=mxrealloc(kn2, (thread+1)*sizeof(char*));
+  kn3=mxrealloc(kn3, (thread+1)*sizeof(char*));
+  c=mxrealloc(c, (thread+1) * sizeof(blf_ctx));
+  CT=mxrealloc(CT, (thread+1) * sizeof(TWOFISH_context));
+  ckey=mxrealloc(ckey, (thread+1) * sizeof(struct cast_key));
+  keyword_given=mxrealloc(keyword_given, (thread+1)*sizeof(char*));
+  algorithm_given=mxrealloc(algorithm_given, (thread+1)*sizeof(int));
   
-  algorithm_given[cur_thread]=algorithm;
-  keyword_given[cur_thread]=mxcalloc(1, get_key_size(algorithm));
-  memmove(keyword_given[cur_thread], key, lenofkey);
-  skey=keyword_given[cur_thread];
+  
+  if (get_key_size(algorithm) < lenofkey) lenofkey=get_key_size(algorithm);
+
+  check_pointer[thread]=1;  
+  algorithm_given[thread]=algorithm;
+  keyword_given[thread]=mxcalloc(1, get_key_size(algorithm));
+  memmove(keyword_given[thread], key, lenofkey);
+  skey=keyword_given[thread];
 
 
    switch(algorithm) {
   
    case DES:
       desinit(0);
-      kn1[cur_thread]=mxmalloc(8*16*sizeof(char));
+      kn1[thread]=mxmalloc(8*16*sizeof(char));
       
-      des_setkey( (void*) kn1[cur_thread], (void *) keyword_given[cur_thread]);
-      return cur_thread;
+      des_setkey( (void*) kn1[thread], (void *) keyword_given[thread]);
+      return thread;
       break;
 
    case SAFER64:
       keyword1=mxmalloc(8);
-      memmove(keyword1, keyword_given[cur_thread], 8);
-      saferkey[cur_thread]=mxmalloc(1 + SAFER_BLOCK_LEN * (1 + 2 * SAFER_MAX_NOF_ROUNDS));
+      memmove(keyword1, keyword_given[thread], 8);
+      saferkey[thread]=mxmalloc(1 + SAFER_BLOCK_LEN * (1 + 2 * SAFER_MAX_NOF_ROUNDS));
 
       Safer_Init_Module();
-      Safer_Expand_Userkey( (unsigned char*) keyword1,(unsigned char*) keyword1, SAFER_SK64_DEFAULT_NOF_ROUNDS, 1, saferkey[cur_thread]);
+      Safer_Expand_Userkey( (unsigned char*) keyword1,(unsigned char*) keyword1, SAFER_SK64_DEFAULT_NOF_ROUNDS, 1, saferkey[thread]);
       /* 8 rounds and secure key schedule */
       free(keyword1);
 
    case CAST128:
-      cast_setkey( &ckey[cur_thread], (void*)keyword_given[cur_thread], lenofkey);
-      return cur_thread;
+      cast_setkey( &ckey[thread], (void*)keyword_given[thread], lenofkey);
+      return thread;
       break; 
 
    case SAFER128:
       Safer_Init_Module();
       keyword1=mxmalloc(8); keyword2=mxmalloc(8);
-      saferkey[cur_thread]=mxmalloc(1 + SAFER_BLOCK_LEN * (1 + 2 * SAFER_MAX_NOF_ROUNDS));
+      saferkey[thread]=mxmalloc(1 + SAFER_BLOCK_LEN * (1 + 2 * SAFER_MAX_NOF_ROUNDS));
       
       memmove(keyword1, skey, 8);
       memmove(keyword2, &skey[8], 8);
-      Safer_Expand_Userkey( (unsigned char*)keyword1, (unsigned char*)keyword2, SAFER_SK128_DEFAULT_NOF_ROUNDS, 1, saferkey[cur_thread]);
+      Safer_Expand_Userkey( (unsigned char*)keyword1, (unsigned char*)keyword2, SAFER_SK128_DEFAULT_NOF_ROUNDS, 1, saferkey[thread]);
       /* 8 rounds and secure key schedule */
       free(keyword1); free(keyword2);
 
-      return cur_thread;
+      return thread;
       break;
       
    case TripleDES:
@@ -212,52 +215,52 @@ word8 twokey[16]; /* 128 bits */
       keyword1=mxcalloc(1,8*sizeof(char)); 
       keyword2=mxcalloc(1,8*sizeof(char)); 
       keyword3=mxcalloc(1,8*sizeof(char));
-      kn1[cur_thread]=mxmalloc(8*16*sizeof(char));
-      kn2[cur_thread]=mxmalloc(8*16*sizeof(char));
-      kn3[cur_thread]=mxmalloc(8*16*sizeof(char));
+      kn1[thread]=mxmalloc(8*16*sizeof(char));
+      kn2[thread]=mxmalloc(8*16*sizeof(char));
+      kn3[thread]=mxmalloc(8*16*sizeof(char));
       
       /* If it returns 1 or -1 the key is too small */
 
-      if (BreakToThree(keyword_given[cur_thread], lenofkey, keyword1, keyword2, keyword3)!=0) {
+      if (BreakToThree(keyword_given[thread], lenofkey, keyword1, keyword2, keyword3)!=0) {
         free(keyword1); free(keyword2); free(keyword3);
-        end_mcrypt(cur_thread);
+        end_mcrypt(thread);
         return (-1);
       }
-      des_setkey( (void*)kn1[cur_thread], keyword1);
-      des_setkey( (void*)kn2[cur_thread], keyword2);
-      des_setkey( (void*)kn3[cur_thread], keyword3);
+      des_setkey( (void*)kn1[thread], keyword1);
+      des_setkey( (void*)kn2[thread], keyword2);
+      des_setkey( (void*)kn3[thread], keyword3);
       free(keyword1); free(keyword2); free(keyword3);
 
-      return cur_thread;
+      return thread;
       break;
        
    case BLOWFISH:
       if (lenofkey<2) {
-          blf_key(&c[cur_thread], (void*) keyword_given[cur_thread],2);
+          blf_key(&c[thread], (void*) keyword_given[thread],2);
       }
       else {
-          blf_key(&c[cur_thread], (void*) keyword_given[cur_thread],lenofkey);
+          blf_key(&c[thread], (void*) keyword_given[thread],lenofkey);
       }
-      return cur_thread;
+      return thread;
       break;
 
    case TWOFISH:
 
       Bzero( twokey, sizeof(twokey));
-      memmove( twokey, keyword_given[cur_thread], 16); /* 128 bit only */
-      twofish_setkey ( &CT[cur_thread], twokey, 16);
+      memmove( twokey, keyword_given[thread], 16); /* 128 bit only */
+      twofish_setkey ( &CT[thread], twokey, 16);
 
-      return cur_thread;
+      return thread;
       break;
 
    case GOST:
       kboxinit();
-      return cur_thread;
+      return thread;
       break;
 
    case ThreeWAY:
    case TEAN:
-      return cur_thread;
+      return thread;
       break;
 
    default:
@@ -395,7 +398,7 @@ int end_mcrypt(int td)
 {
 
 free(keyword_given[td]); 
-keyword_given[td]=NULL;
+check_pointer[td]=0;
 
    switch(algorithm_given[td]) {
   
@@ -437,25 +440,19 @@ return 0;
 
 int init_mcrypt_ecb(const int algorithm, void * key, int lenofkey)
 {
-
 int x;
 
 #ifdef HAVE_LIBPTHREAD
 pthread_mutex_lock(&counter_cur_thread);
 #endif 
 
-cur_thread++;
-
-  if (cur_thread < MAX_THREADS && finished_threads==0) {
-
-  }
-  else {
+  if (cur_thread == MAX_THREADS) {
      cur_thread=0;
-     finished_threads=1;
-  
-     while (keyword_given[cur_thread]!=NULL && cur_thread <= MAX_THREADS) {
-       cur_thread++;
-     }  
+  }  
+
+  while (check_pointer[cur_thread]==1 && cur_thread < MAX_THREADS) {
+    cur_thread++;
+  }  
   
      if (cur_thread >= MAX_THREADS) {
         #ifdef HAVE_LIBPTHREAD
@@ -463,9 +460,8 @@ cur_thread++;
         #endif
         return -1;
      }
-  }
 
-x=init_mcrypt(algorithm, key, lenofkey);
+ x=init_mcrypt(algorithm, cur_thread, key, lenofkey);
 
  if (x<0) {
    end_mcrypt(cur_thread);
@@ -542,7 +538,7 @@ pthread_mutex_lock(&counter_cur_thread);
 #endif
 cur_thread++;
 
-if (cur_thread < MAX_THREADS && finished_threads==0) {
+if (cur_thread < MAX_THREADS && check_pointer[cur_thread]==0) {
 
 /* for cbc */
   previous_ciphertext=mxrealloc(previous_ciphertext, (cur_thread+1) * sizeof(word32*));
@@ -553,10 +549,10 @@ if (cur_thread < MAX_THREADS && finished_threads==0) {
 }
 else {
   cur_thread=0;
-  finished_threads=1;
-  while (keyword_given[cur_thread]!=NULL && cur_thread <= MAX_THREADS) {
+  while (check_pointer[cur_thread]==1 && cur_thread <= MAX_THREADS) {
    cur_thread++;
   }  
+
   if (cur_thread >= MAX_THREADS) {
     #ifdef HAVE_LIBPTHREAD
     pthread_mutex_unlock(&counter_cur_thread);
@@ -573,7 +569,7 @@ else {
 
 /* End cbc */
 
-x=init_mcrypt(algorithm, key, lenofkey);
+x=init_mcrypt(algorithm, cur_thread, key, lenofkey);
 
  if (x<0) {
    end_mcrypt_cbc(cur_thread);
@@ -668,7 +664,7 @@ pthread_mutex_lock(&counter_cur_thread);
 cur_thread++;
 
 
-if (cur_thread < MAX_THREADS && finished_threads==0) {
+if (cur_thread < MAX_THREADS && check_pointer[cur_thread]==0) {
 
 /* for cfb */
   s_register=mxrealloc(s_register, (cur_thread+1) * sizeof(char*));
@@ -679,10 +675,11 @@ if (cur_thread < MAX_THREADS && finished_threads==0) {
 }
 else {
   cur_thread=0;
-  finished_threads=1;
-  while (keyword_given[cur_thread]!=NULL && cur_thread <= MAX_THREADS) {
+
+  while (check_pointer[cur_thread]==1 && cur_thread <= MAX_THREADS) {
    cur_thread++;
   }  
+
   if (cur_thread >= MAX_THREADS) {
     #ifdef HAVE_LIBPTHREAD
     pthread_mutex_unlock(&counter_cur_thread);
@@ -702,7 +699,7 @@ else {
 
 /* End cfb */
 
-x=init_mcrypt( algorithm, key, lenofkey);
+x=init_mcrypt( algorithm, cur_thread, key, lenofkey);
 
  if (x<0) {
    end_mcrypt_cfb(cur_thread);
@@ -771,6 +768,137 @@ int mdecrypt_cfb(int td, void *plaintext,int len) /* plaintext is 1 byte (8bit c
 }
 
 
+/* OFB MODE */
+
+int end_mcrypt_ofb(int td)
+{
+
+free(s_register[td]);
+free(enc_s_register[td]);
+free(enc_sd_register[td]);
+free(sd_register[td]);
+
+return end_mcrypt(td);
+
+}
+
+
+int init_mcrypt_ofb(const int algorithm, void * key, int lenofkey, void *IV)
+{
+int x;
+
+#ifdef HAVE_LIBPTHREAD
+pthread_mutex_lock(&counter_cur_thread);
+#endif
+cur_thread++;
+
+
+if (cur_thread < MAX_THREADS && check_pointer[cur_thread]==0) {
+
+/* for ofb */
+  s_register=mxrealloc(s_register, (cur_thread+1) * sizeof(char*));
+  enc_s_register=mxrealloc(enc_s_register, (cur_thread+1) * sizeof(char*));  
+  enc_sd_register=mxrealloc(enc_sd_register, (cur_thread+1) * sizeof(char*));  
+  sd_register=mxrealloc(sd_register, (cur_thread+1) * sizeof(char*));
+/* end ofb */
+}
+else {
+  cur_thread=0;
+
+  while (check_pointer[cur_thread]==1 && cur_thread <= MAX_THREADS) {
+   cur_thread++;
+  }  
+
+  if (cur_thread >= MAX_THREADS) {
+    #ifdef HAVE_LIBPTHREAD
+    pthread_mutex_unlock(&counter_cur_thread);
+    #endif
+    return -1;
+  }
+}
+
+
+/* For cfb */
+  s_register[cur_thread]=mxmalloc( get_block_size(algorithm));
+  sd_register[cur_thread]=mxmalloc( get_block_size(algorithm));
+  enc_s_register[cur_thread]=mxmalloc( get_block_size(algorithm));
+  enc_sd_register[cur_thread]=mxmalloc( get_block_size(algorithm));
+  memmove(sd_register[cur_thread], IV, get_block_size(algorithm));
+  memmove(s_register[cur_thread], IV, get_block_size(algorithm));  
+
+/* End cfb */
+
+x=init_mcrypt( algorithm, cur_thread, key, lenofkey);
+
+ if (x<0) {
+   end_mcrypt_cfb(cur_thread);
+   #ifdef HAVE_LIBPTHREAD
+   pthread_mutex_unlock(&counter_cur_thread);
+   #endif
+   return -1;
+ }
+ else {
+   #ifdef HAVE_LIBPTHREAD
+   pthread_mutex_unlock(&counter_cur_thread);
+   #endif
+   return x;
+ }
+}
+
+int mcrypt_ofb(int td, void *plaintext,int len) /* plaintext is 1 byte (8bit cfb) */
+{
+  char* plain=plaintext;
+  int i,j,blocksize=get_block_size(algorithm_given[td]);;
+
+ for (j=0;j<len;j++) {
+  
+  memmove(enc_s_register[td], s_register[td], blocksize);
+  
+  mcrypt(td, enc_s_register[td]);
+
+  plain[j] ^= enc_s_register[td][0];
+  
+/* Shift the register */
+  for (i=0 ; i< (blocksize-1) ; i++) 
+    s_register[td][i]=s_register[td][i+1];
+  
+  s_register[td][blocksize-1]=enc_s_register[td][0];
+
+
+ }
+
+  return 0;
+
+}
+
+
+int mdecrypt_ofb(int td, void *plaintext,int len) /* plaintext is 1 byte (8bit cfb) */
+{
+  char* plain=plaintext;
+  int i,j,blocksize=get_block_size(algorithm_given[td]);;
+  
+ for (j=0;j<len;j++) {
+ 
+  memmove(enc_sd_register[td], sd_register[td], blocksize);
+
+  mcrypt(td, enc_sd_register[td]);
+
+/* Shift the register */
+  for (i=0 ; i<(blocksize-1) ; i++) 
+    sd_register[td][i]=sd_register[td][i+1];
+  
+  sd_register[td][blocksize-1]=enc_sd_register[td][0];
+
+  plain[j] ^= enc_sd_register[td][0];
+
+ }
+
+  return 0;
+}
+
+
+/* Extra functions */
+
 char* get_algorithms_name(int algorithm) {
 
 char *x=mxmalloc(12);
