@@ -14,6 +14,13 @@
 # include "functions.h"
 #endif
 
+#ifdef BZIP2
+# define ZIP
+#endif
+
+#ifdef GZIP
+# define ZIP
+#endif
 
 #ifdef HAVE_SIGNAL_H
 # include <signal.h>
@@ -65,6 +72,7 @@
 #define CBC 0
 #define ECB 1
 #define CFB 2
+#define OFB 3
 
 #define ENCRYPT 0
 #define DECRYPT 1
