@@ -13,7 +13,7 @@
 #include <enigma.h>
 #include <bzero.h>
 
-#define ECHO 010
+#define E_ECHO 010
 #define ROTORSZ 256
 #define MASK 0377
 char	t1[ROTORSZ];
