@@ -4,7 +4,7 @@
 #endif
 #include <environ.h>
 #include <extra.h>
-#include <xmemory.h>
+#include <xmalloc.h>
 
 int check_env () {
 
@@ -19,7 +19,7 @@ char ** get_env_key() {
 char **tmp;
 
  if (getenv(MCRYPT_KEY)!=NULL) {
-  tmp=mxmalloc(sizeof(char*));
+  tmp= xmalloc(sizeof(char*));
   tmp[0]=getenv(MCRYPT_KEY);
   return tmp;
  }
