@@ -98,8 +98,14 @@ int show_mode(int ende, int stype,int smode,char *output,int bitmode)
 char *x=get_algorithms_name(stype);
 
 
+ if (stype==CRYPT) {
+  x=malloc(12);
+  strcpy(x,"Unix Crypt");
+ }
+
  fprintf (stderr,_( "Algorithm: %s\n"),x);
  free(x); 
+ 
    
  if (stype!=CRYPT) {
    switch(smode) {
@@ -112,6 +118,9 @@ char *x=get_algorithms_name(stype);
     case CFB:
      fprintf (stderr,_( "Mode: CFB\n"));
      break;
+    case OFB:
+     fprintf (stderr,_( "Mode: OFB\n"));
+     break;     
    }
  }
 
@@ -158,34 +167,38 @@ int check_file_head(char * filename, unsigned int *algorithm, unsigned int *mode
 FILE *fstream;
 char buf[6];
 
+
  if (stream_flag==TRUE) {
   fstream=(FILE *) stdin;
  }
  else {
   fstream=fopen(filename,"r");
-  if (read_lock( fileno((FILE*)fstream))==-1) return (-1);
+  if (read_lock( fileno((FILE*)fstream))==-1) {
+    return (-1);
+  }
  } 
 
  fread(buf, 1, 6, fstream);
 
  if (buf[0]=='\0' && buf[1]=='m' && buf[2]=='\0') {
-  /* headers are ok */
+      /* if headers are ok */
 
   *algorithm=(unsigned int) buf[3];
   *mode=(unsigned int) buf[4]; 
   *bit7_flag=(unsigned int) buf[5];
 
-/* 6 To 9 are crc32 */
+      /* 6 To 9 are crc32 */
 
-  if (stream_flag==FALSE) { fflush(fstream); unlock( fileno((FILE*)fstream)); fclose(fstream); }
+  if (stream_flag==FALSE) 
+     { fflush(fstream); unlock( fileno((FILE*)fstream)); fclose(fstream); }
   return 0;
  }  
  else { /* No headers present */
+
    if (stream_flag==FALSE) { 
     fflush(fstream);
     unlock( fileno((FILE*)fstream));
     fclose(fstream);
-    return 1;
    }
    else { /* restart the stream so we will not loose important data */
      if (fseek( fstream, 0, SEEK_SET)==-1) {
@@ -193,13 +206,10 @@ char buf[6];
        fprintf(stderr,_("Unable to get algorithm information. Use the --bare flag and specify the algorithm manualy.\n"));
        exit(-1);
      }
-     else {
-       return 1;
-     }
-     
    }
- }
 
+   return 1;
+ }
 }
 
 
