@@ -101,6 +101,8 @@ void gaa_help()
 	__gaa_helpsingle('F', "force", "", "forces output to stdout.");
 	__gaa_helpsingle('r', "random", "", "Use real random data (if your system supports it).");
 	__gaa_helpsingle('b', "bare", "", "Do not keep algorithm information in the encrypted""file.");
+	__gaa_helpsingle('z', "gzip", "", "Compress before encryption.");
+	__gaa_helpsingle('p', "bzip", "", "Compress before encryption.");
 	__gaa_helpsingle('d', "decrypt", "", "decrypts.");
 	__gaa_helpsingle('x', "hex", "", "Keyword is in hex.");
 	__gaa_helpsingle('i', "bit", "", "keyword bit mode (7 or 8).");
@@ -128,38 +130,42 @@ typedef struct _gaainfo gaainfo;
 
 struct _gaainfo
 {
-#line 62 "mcrypt.gaa"
+#line 68 "mcrypt.gaa"
 	char **input;
-#line 61 "mcrypt.gaa"
+#line 67 "mcrypt.gaa"
 	int size;
-#line 53 "mcrypt.gaa"
+#line 59 "mcrypt.gaa"
 	char *mode;
-#line 50 "mcrypt.gaa"
+#line 56 "mcrypt.gaa"
 	char **keys;
-#line 49 "mcrypt.gaa"
+#line 55 "mcrypt.gaa"
 	int keysize;
-#line 43 "mcrypt.gaa"
+#line 49 "mcrypt.gaa"
 	char *algorithm;
-#line 40 "mcrypt.gaa"
+#line 46 "mcrypt.gaa"
 	char *config_file;
-#line 39 "mcrypt.gaa"
+#line 45 "mcrypt.gaa"
 	int config;
-#line 36 "mcrypt.gaa"
+#line 42 "mcrypt.gaa"
 	char *keyfile;
-#line 32 "mcrypt.gaa"
+#line 38 "mcrypt.gaa"
 	int sunlink_flag;
-#line 28 "mcrypt.gaa"
+#line 34 "mcrypt.gaa"
 	int unlink_flag;
-#line 25 "mcrypt.gaa"
+#line 31 "mcrypt.gaa"
 	int bit;
-#line 22 "mcrypt.gaa"
+#line 28 "mcrypt.gaa"
 	int hexflag;
-#line 19 "mcrypt.gaa"
+#line 25 "mcrypt.gaa"
 	int ed_specified;
-#line 18 "mcrypt.gaa"
+#line 24 "mcrypt.gaa"
 	int ein;
-#line 17 "mcrypt.gaa"
+#line 23 "mcrypt.gaa"
 	int din;
+#line 20 "mcrypt.gaa"
+	int bzipflag;
+#line 17 "mcrypt.gaa"
+	int gzipflag;
 #line 13 "mcrypt.gaa"
 	int bare_flag;
 #line 10 "mcrypt.gaa"
@@ -222,7 +228,7 @@ int gaa_error = 0;
 #define GAA_MULTIPLE_OPTION     3
 
 #define GAA_REST                0
-#define GAA_NB_OPTION           18
+#define GAA_NB_OPTION           20
 #define GAAOPTID_license	1
 #define GAAOPTID_version	2
 #define GAAOPTID_help	3
@@ -236,11 +242,13 @@ int gaa_error = 0;
 #define GAAOPTID_bit	11
 #define GAAOPTID_hex	12
 #define GAAOPTID_decrypt	13
-#define GAAOPTID_bare	14
-#define GAAOPTID_random	15
-#define GAAOPTID_force	16
-#define GAAOPTID_quiet	17
-#define GAAOPTID_verbose	18
+#define GAAOPTID_bzip	14
+#define GAAOPTID_gzip	15
+#define GAAOPTID_bare	16
+#define GAAOPTID_random	17
+#define GAAOPTID_force	18
+#define GAAOPTID_quiet	19
+#define GAAOPTID_verbose	20
 
 #line 168 "gaa.skel"
 
@@ -487,6 +495,14 @@ struct GAAOPTION_decrypt
 {
 };
 
+struct GAAOPTION_bzip 
+{
+};
+
+struct GAAOPTION_gzip 
+{
+};
+
 struct GAAOPTION_bare 
 {
 };
@@ -554,6 +570,8 @@ int gaa_get_option_num(char *str, int status)
 			GAA_CHECK1STR("u", GAAOPTID_unlink);
 			GAA_CHECK1STR("x", GAAOPTID_hex);
 			GAA_CHECK1STR("d", GAAOPTID_decrypt);
+			GAA_CHECK1STR("p", GAAOPTID_bzip);
+			GAA_CHECK1STR("z", GAAOPTID_gzip);
 			GAA_CHECK1STR("b", GAAOPTID_bare);
 			GAA_CHECK1STR("r", GAAOPTID_random);
 			GAA_CHECK1STR("F", GAAOPTID_force);
@@ -576,6 +594,8 @@ int gaa_get_option_num(char *str, int status)
 			GAA_CHECKSTR("bit", GAAOPTID_bit);
 			GAA_CHECKSTR("hex", GAAOPTID_hex);
 			GAA_CHECKSTR("decrypt", GAAOPTID_decrypt);
+			GAA_CHECKSTR("bzip", GAAOPTID_bzip);
+			GAA_CHECKSTR("gzip", GAAOPTID_gzip);
 			GAA_CHECKSTR("bare", GAAOPTID_bare);
 			GAA_CHECKSTR("random", GAAOPTID_random);
 			GAA_CHECKSTR("force", GAAOPTID_force);
@@ -606,6 +626,8 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
 	struct GAAOPTION_bit GAATMP_bit;
 	struct GAAOPTION_hex GAATMP_hex;
 	struct GAAOPTION_decrypt GAATMP_decrypt;
+	struct GAAOPTION_bzip GAATMP_bzip;
+	struct GAAOPTION_gzip GAATMP_gzip;
 	struct GAAOPTION_bare GAATMP_bare;
 	struct GAAOPTION_random GAATMP_random;
 	struct GAAOPTION_force GAATMP_force;
@@ -633,21 +655,21 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
     {
 	case GAAOPTID_license:
 	OK = 0;
-#line 59 "mcrypt.gaa"
+#line 65 "mcrypt.gaa"
 { mcrypt_license(); exit(0); ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_version:
 	OK = 0;
-#line 58 "mcrypt.gaa"
+#line 64 "mcrypt.gaa"
 { mcrypt_version(); exit(0); ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_help:
 	OK = 0;
-#line 57 "mcrypt.gaa"
+#line 63 "mcrypt.gaa"
 { usage(); exit(0); ;};
 
 		return GAA_OK;
@@ -657,7 +679,7 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
 		GAA_TESTMOREARGS;
 		GAA_FILL(GAATMP_mode.arg1, gaa_getstr, GAATMP_mode.size1);
 		gaa_index++;
-#line 54 "mcrypt.gaa"
+#line 60 "mcrypt.gaa"
 { gaaval->mode = GAATMP_mode.arg1 ;};
 
 		return GAA_OK;
@@ -665,7 +687,7 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
 	case GAAOPTID_key:
 	OK = 0;
 		GAA_LIST_FILL(GAATMP_key.arg1, gaa_getstr, char*, GAATMP_key.size1);
-#line 51 "mcrypt.gaa"
+#line 57 "mcrypt.gaa"
 { gaaval->keys = GAATMP_key.arg1; gaaval->keysize = GAATMP_key.size1 ;};
 
 		return GAA_OK;
@@ -675,7 +697,7 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
 		GAA_TESTMOREARGS;
 		GAA_FILL(GAATMP_algorithm.arg1, gaa_getstr, GAATMP_algorithm.size1);
 		gaa_index++;
-#line 44 "mcrypt.gaa"
+#line 50 "mcrypt.gaa"
 { gaaval->algorithm = GAATMP_algorithm.arg1 ;};
 
 		return GAA_OK;
@@ -685,7 +707,7 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
 		GAA_TESTMOREARGS;
 		GAA_FILL(GAATMP_config.arg1, gaa_getstr, GAATMP_config.size1);
 		gaa_index++;
-#line 41 "mcrypt.gaa"
+#line 47 "mcrypt.gaa"
 { gaaval->config=1; if (GAATMP_config.size1==1) gaaval->config_file = GAATMP_config.arg1 ;};
 
 		return GAA_OK;
@@ -695,21 +717,21 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
 		GAA_TESTMOREARGS;
 		GAA_FILL(GAATMP_keyfile.arg1, gaa_getstr, GAATMP_keyfile.size1);
 		gaa_index++;
-#line 37 "mcrypt.gaa"
+#line 43 "mcrypt.gaa"
 { gaaval->keyfile = GAATMP_keyfile.arg1 ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_sunlink:
 	OK = 0;
-#line 33 "mcrypt.gaa"
+#line 39 "mcrypt.gaa"
 { gaaval->sunlink_flag = 1 ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_unlink:
 	OK = 0;
-#line 29 "mcrypt.gaa"
+#line 35 "mcrypt.gaa"
 { gaaval->unlink_flag = 1 ;};
 
 		return GAA_OK;
@@ -719,23 +741,37 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
 		GAA_TESTMOREARGS;
 		GAA_FILL(GAATMP_bit.arg1, gaa_getint, GAATMP_bit.size1);
 		gaa_index++;
-#line 26 "mcrypt.gaa"
+#line 32 "mcrypt.gaa"
 { gaaval->bit = GAATMP_bit.arg1 ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_hex:
 	OK = 0;
-#line 23 "mcrypt.gaa"
+#line 29 "mcrypt.gaa"
 { gaaval->hexflag = 1 ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_decrypt:
 	OK = 0;
-#line 20 "mcrypt.gaa"
+#line 26 "mcrypt.gaa"
 { gaaval->ed_specified=1; gaaval->din = 1; gaaval->ein=0; ;};
 
+		return GAA_OK;
+		break;
+	case GAAOPTID_bzip:
+	OK = 0;
+#line 21 "mcrypt.gaa"
+{ gaaval->bzipflag = 1 ;};
+
+		return GAA_OK;
+		break;
+	case GAAOPTID_gzip:
+	OK = 0;
+#line 18 "mcrypt.gaa"
+{ gaaval->gzipflag = 1 ;};
+
 		return GAA_OK;
 		break;
 	case GAAOPTID_bare:
@@ -775,7 +811,7 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
 		break;
 	case GAA_REST:
 		GAA_OPTIONALLIST_FILL(GAAREST_tmp.arg1, gaa_getstr, char*, GAAREST_tmp.size1);
-#line 63 "mcrypt.gaa"
+#line 69 "mcrypt.gaa"
 { gaaval->input = GAAREST_tmp.arg1; gaaval->size = GAAREST_tmp.size1 ;};
 
 		return GAA_OK;
@@ -803,10 +839,10 @@ int gaa(int argc, char **argv, gaainfo *gaaval)
     if(inited == 0)
     {
 
-#line 65 "mcrypt.gaa"
+#line 71 "mcrypt.gaa"
 { gaaval->force=0; gaaval->quiet=1; gaaval->real_random_flag=0; gaaval->config=0;
        gaaval->bare_flag = 0; gaaval->din = 0; gaaval->ein=1; gaaval->hexflag = 0; gaaval->unlink_flag = 0;
-       gaaval->bit = 8; gaaval->keysize=0;
+       gaaval->bit = 8; gaaval->keysize=0; gaaval->gzipflag=0; gaaval->bzipflag=0;
        gaaval->sunlink_flag = 0; gaaval->keyfile=NULL; gaaval->algorithm=NULL; gaaval->keys=NULL;
        gaaval->config_file=NULL; gaaval->mode=NULL; gaaval->input=NULL; gaaval->ed_specified=0 ;};
 
@@ -880,6 +916,7 @@ int gaa(int argc, char **argv, gaainfo *gaaval)
     }
 if(gaa_processing_file == 0)
 {
+	GAA_INCOMP("zp");
 	GAA_INCOMP("Vq");
 	GAA_INCOMP("kf");
 	GAA_INCOMP("us");
@@ -1027,6 +1064,6 @@ int gaa_file(char *name, gaainfo *gaaval)
     }
 
     rval = gaa(argc, argv, gaaval);
-    gaa_processing_file = 1;
+    gaa_processing_file = 0;
     return rval;
 }
