@@ -75,6 +75,11 @@ readpass(FILE *fp)
 
 	cp = input;
 	ap = asterix;
+
+#ifdef HAVE_DEV_RANDOM
+        open_rand(0); /* We do not need much randomness here */
+#endif
+        
 	while (read(fileno(fp), &c, 1)) {
 		switch (c) {
 		case '\n':
@@ -94,7 +99,11 @@ readpass(FILE *fp)
 			break;
 		default:
 			*cp++ = c;
+#ifdef HAVE_DEV_RANDOM
+			*ap++ = (get_o_rand8() % 4)+1;
+#else
 			*ap++ = (RAND8 % 4)+1;
+#endif			
 			for (i = 0; i < (*(ap-1)); i++)
 				putc('*', stderr);
 			break;
@@ -103,9 +112,15 @@ readpass(FILE *fp)
 		if (cp == input + MAXLEN)
 			break;
 	}
+
+#ifdef HAVE_DEV_RANDOM
+	close_rand(); /* close /dev/urandom */
+#endif	
+
 endwhile:
 	*cp = 0;
 	putc('\n', stderr);
+        fflush(stderr);
 	return input;
 }
 #else
