@@ -22,10 +22,10 @@
  *    along with this program; if not, write to the Free Software
  *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
+
 
 #include <mcrypt.h>
 #include <xmalloc.h>
-#include <xmemory.h>
 #include <bzero.h>
 #include <extra.h>
 #include <bits.h>
@@ -40,6 +40,7 @@
 #include <gaa.h>
 #include <errors.h>
 #include <environ.h>
+#include <swap.h>
 
  char tmperr[128];
  unsigned int bit7_flag, hexflag=FALSE;
@@ -51,6 +52,10 @@
  int sunlink_flag=FALSE; 
  int bare_flag=FALSE, real_random_flag=FALSE;
  int cleanDelete=FALSE;
+#ifdef ZIP
+ int gzipflag=FALSE;
+ int bzipflag=FALSE;
+#endif
  char *outfile=0;
 
 void usage(char *name)
@@ -69,7 +74,7 @@ void usage(char *name)
                               Legal parameters are: 3way, des, 3des, gost, tea,\n\
                               safer64, safer128, cast128, crypt, twofish\n\
                               and blowfish. (Default is blowfish)\n\
- -m --mode MODE               specify the mode (cfb, ecb or cbc).\n\
+ -m --mode MODE               specify the mode (cfb, ofb, ecb or cbc).\n\
                               (Default is cbc)\n\
  -k --key KEY1, KEY2, ...     keyword(s) to use.\n\
  -f --keyfile FILENAME        file to read the keyword from.\n\
@@ -78,14 +83,26 @@ void usage(char *name)
  -u --unlink                  unlink the input file after encryption/decryption\n\
  -s --sunlink                 unlink and destroy the input file after\n\
                               encryption/decryption\n\
- -r --random                  Use real random data (if your system supports it).\n\
  -L --license                 displays license information.\n\
  -q --quiet                   suppress some non critical warnings.\n\
  -V --verbose                 some more information is displayed.\n\
  -b --bare                    Do not keep algorithm information in the encrypted\n\
                               file.\n\
- -F --force                   forces output to stdout.\n\
-\n\
+ -F --force                   forces output to stdout.\n"));
+
+#ifdef HAVE_DEV_RANDOM
+fprintf(stderr,_(" -r --random                  Use real random data.\n"));
+#endif
+
+#ifdef GZIP
+fprintf(stderr,_(" -z --gzip                    use gzip to compress files before encryption.\n"));
+#endif
+
+#ifdef BZIP2
+fprintf(stderr,_(" -p --bzip2                   use bzip2 to compress files before encryption.\n"));
+#endif
+
+fprintf(stderr,_("\n\
  -h --help                    prints this help\n\
  -v --version                 prints the version number\n\
 \n\
@@ -111,7 +128,7 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n\n"));
 
 void mcrypt_version() {
 
-  fprintf (stderr, _("Mini-crypt v.%s\n"),VERSION);
+  fprintf (stderr, _("Mini-crypt v.%s (%s-%s-%s)\n"), VERSION, T_CPU, T_VENDOR, T_OS);
   fprintf (stderr, _("Copyright (C) 1998,1999 Nikos Mavroyanopoulos (nmav@i-net.paiko.gr)\n"));
 
 }
@@ -204,11 +221,11 @@ int main(int argc,char **argv)
 /* Check again the command line variables 
  * This will be uncommented when gaa is corrected.
  */
-/*
+
         if((v = gaa(argc, argv, &info)) != -1) {
           err_quit(_("Error in the arguments.\n"));
         }
-*/
+
 
 /* Examine how we were called */
    if (strstr(argv[0],"decrypt")!=NULL) {
@@ -251,7 +268,10 @@ int main(int argc,char **argv)
                 sunlink_flag=info.sunlink_flag;
 		quiet=info.quiet;
                 hexflag=info.hexflag;
-
+#ifdef ZIP
+                gzipflag=info.gzipflag;
+                bzipflag=info.bzipflag;
+#endif
                 if (info.bit==7) {
                   if (hexflag==TRUE) 
                     err_quit(_("Hex mode and 7bit mode are mutually exclusive\n"));
@@ -302,11 +322,16 @@ int main(int argc,char **argv)
                               mode=CFB;
 	  	            }
 	  	            else {
-                              mode=CBC; /* Default */
+                               if (strncasecmp(modet,"OFB",3)==0) {
+                                 mode=OFB;
+                               }
+	  	               else {
+                                 mode=CBC; /* Default */
 
                               sprintf(tmperr, _("No such mode: %s\n"),modet);
                               err_crit (tmperr);
                               err_crit (_("Assuming CBC.\n"));
+                               }
 	  	            }
 			  }
                         
@@ -434,17 +459,25 @@ for(i=0;i<file_count;i++) {
 
 #ifdef HAVE_STAT
     if ( is_normal_file( file[i]) == FALSE && stream_flag==FALSE ) {
-      sprintf(tmperr, _("%s: %s is not a regular file. Skipping...\n"), argv[0], file[i]);
-      err_crit (tmperr);
-      outfile=NULL;
-      continue; /* next */
+        sprintf(tmperr, _("%s: %s is not a regular file. Skipping...\n"), argv[0], file[i]);
+        err_crit (tmperr);
+        outfile=NULL;
+        continue; /* next */
     }
 #endif
 
+#ifdef ZIP
+            if (stream_flag==TRUE) {
+              if (gzipflag==TRUE || bzipflag==TRUE) {
+                err_quit(_("--bzip2 or --gzip parameters cannot be used when dealing with streams.\n"));
+              }
+            }
+#endif
+
          /* Check how we were called */
 	if (din==TRUE) {
             if (stream_flag!=TRUE) dinfile=file[i];
-            if ( (isatty(fileno( (FILE *)(stdin) ) ) == 1) && (stream_flag==TRUE) && (force==0) ) { /* not a tty */        
+            if ( (isatty(fileno( (FILE *)(stdin) ) ) == 1) && (stream_flag==TRUE) && (force==0) ) { /* not a tty */
               sprintf(tmperr, _("%s: Encrypted data will not be read from a terminal.\n"),argv[0]);
               err_crit (tmperr);
               err_quit (_("Redirect the input instead.\nUse the --help parameter for more help.\n"));
@@ -462,10 +495,11 @@ for(i=0;i<file_count;i++) {
 
         /* If no streams make the extensions */
        if(stream_flag!=TRUE) {
+
 	 if (din==TRUE) {
 
             y=strlen(dinfile);
-            /* If the file finished in .enc remove it */
+            /* If the file has the .enc suffix, then remove it */
              
             if (dinfile[y-1]=='c' && dinfile[y-2]=='n' && dinfile[y-3]=='e' && dinfile[y-4]=='.') {
 
@@ -489,13 +523,16 @@ for(i=0;i<file_count;i++) {
             }
             else { /* append .dec */
 
+             cleanDelete=TRUE;
 	     outfile=xcalloc(y+5,1);
              strncpy(outfile,dinfile,y);
              strcat(outfile,".dec");
+
+             
             }
          }
 	 else { /* encryption- append .enc */
-	    outfile= xcalloc(strlen(einfile)+5,1);
+	    outfile= xcalloc(strlen(einfile)+5+4,1);
 	    strcpy(outfile,einfile);
             /* if file has already the .enc ignore it */
             if (strstr( outfile, ".enc")!=NULL) {
@@ -504,6 +541,12 @@ for(i=0;i<file_count;i++) {
               continue;
             }
 
+#ifdef ZIP
+            if (stream_flag==FALSE) {
+              if (gzipflag==TRUE) strcat(outfile, ".gz");
+              if (bzipflag==TRUE) strcat(outfile, ".bz2");
+            }
+#endif
 	    strcat(outfile,".enc");
 #ifdef HAVE_STAT
               /* But if it exists exit */
@@ -536,17 +579,19 @@ for(i=0;i<file_count;i++) {
 
            if (bare_flag==FALSE) {
              x=check_file_head(file[i],&type,&mode, &bit7_flag, stream_flag);
-             if (x!=FALSE) {
+             if (x!=0) {
+
                if (x==-1) {
                  type=NO_TYPE; /* Lock failed */
                }
                else { /* Unix Crypt */
-                if (used_algo==FALSE) {
+                if (used_algo==FALSE || bare_flag==FALSE) {
                  sprintf(tmperr, _("%s: no headers. Assuming encrypted file with Unix crypt.\n"),argv[0]);
                  err_crit (tmperr);
                  fflush(stderr);
                  type=CRYPT;
                 }
+
                }
              }
            }
@@ -555,6 +600,7 @@ for(i=0;i<file_count;i++) {
                         case CBC:
                         case ECB:
                         case CFB:
+                        case OFB:
 
   			      switch (type) {
 			        case DES:
@@ -634,6 +680,8 @@ for(i=0;i<file_count;i++) {
                     case CBC:
                     case ECB:
                     case CFB:
+                    case OFB:
+
 
 		      switch (type) {
 		        case DES:
@@ -711,11 +759,16 @@ for(i=0;i<file_count;i++) {
     Bzero(argv[i],strlen(argv[i]));
   }
 
-  return (return_val);
+  if (return_val != 0) {
+       return 1;
+  }
+  else {
+       return 0;
+  }
 
 }
 
-
+
 
 
 /* UNIX CRYPT Encryption */
@@ -729,22 +782,25 @@ char keyword[13];
 char *tmp;
 FILE *TOF;
 FILE *FROMF;
+char* command=NULL;
+
 
 /* Crypt does not care about these 
- * It's purpose is to be compatible with the unix crypt not safe.
+ * It's purpose is to be compatible with the unix crypt which is not safe.
  * It isn't a block algorithm after all.
  */
+  bit7_flag=BITS8;
 
-bit7_flag=BITS8;
-
-
+  
 /* open files */
-if (stream_flag==TRUE) {
+  if (stream_flag==TRUE) {
 	FROMF=stdin;
 	TOF=stdout;
-}
-else {
-
+  }
+  else {
+#ifdef ZIP
+    if (gzipflag==FALSE && bzipflag==FALSE) {
+#endif
 	FROMF=fopen(fromfile,"rb");
 	if (FROMF==NULL) {perror("fopen");return 1;}
 	if (read_lock( fileno((FILE*)FROMF))==-1) return 1;
@@ -752,7 +808,31 @@ else {
 	TOF=fopen(tofile,"wb");
 	if (TOF==NULL) {perror("fopen");return -1;}
 	if (write_lock( fileno((FILE*)TOF))==-1) return -1;
-}
+#ifdef ZIP
+    }
+    else {  
+           /* command if using gzip */
+        if (gzipflag==TRUE) {
+          command=malloc(strlen(GZIP)+strlen(fromfile)+5);
+          strcpy(command, GZIP);
+        }
+        if (bzipflag==TRUE) {
+          command=malloc(strlen(BZIP2)+strlen(fromfile)+5);
+          strcpy(command, BZIP2);
+        }
+        strcat(command, " -c ");
+        strcat(command, fromfile);
+
+    	FROMF=popen(command ,"r");
+	if (FROMF==NULL) {perror("fopen");return 1;}
+	
+	TOF=fopen(tofile,"wb");
+	if (TOF==NULL) {perror("fopen");return -1;}
+	if (write_lock( fileno((FILE*)TOF))==-1) return -1;
+    
+    }
+#endif
+  }
 
          Bzero(keyword,sizeof(keyword));
 
@@ -766,6 +846,7 @@ else {
           memmove(keyword, tmp, len);
           Bzero(tmp, strlen(tmp));
 
+
 	}
 	else {
            if (strlen(key) > 8) {
@@ -826,14 +907,7 @@ return 0;
 
 
 
-
-
-
-
-
-
-
-
+
 /* General Encryption */
 
 int encrypt_general(int algorithm, char *fromfile, char *tofile, char *key)
@@ -849,6 +923,7 @@ FILE *FROMF;
 int keysize=get_key_size(algorithm);
 int blocksize=get_block_size(algorithm);
 int td;
+char *command=NULL;
 
 word32 *IV=0;
 word32 *ciphertext=xmalloc(blocksize);
@@ -858,7 +933,7 @@ FILE * TOF;
  write_lock( fileno((FILE*)TOF));
         
 
-if (mode==CBC || mode==CFB) {
+if (mode==CBC || mode==CFB || mode==OFB) {
  IV=xmalloc(blocksize);
  /* Initialisation Vector */
 
@@ -881,14 +956,40 @@ if (mode==CBC || mode==CFB) {
 	RTOF=stdout;
   }
   else {
-	FROMF=fopen(fromfile,"rb");
-	if (FROMF==NULL) {perror("fopen");return 1;}
-        if (read_lock( fileno((FILE*)FROMF))==-1) return 1;
-        
-	RTOF=fopen(tofile,"wb");
-	if (RTOF==NULL) {perror("fopen");return -1;}
-        if (write_lock( fileno((FILE*)RTOF))==-1) return -1;
+#ifdef ZIP
+        if (gzipflag==FALSE && bzipflag==FALSE) {
+#endif        
+  	  FROMF=fopen(fromfile,"rb");
+	  if (FROMF==NULL) {perror("fopen");return 1;}
+          if (read_lock( fileno((FILE*)FROMF))==-1) return 1;
         
+	  RTOF=fopen(tofile,"wb");
+	  if (RTOF==NULL) {perror("fopen");return -1;}
+          if (write_lock( fileno((FILE*)RTOF))==-1) return -1;
+#ifdef ZIP
+        }
+        else {
+           /* command if using gzip */
+          if (gzipflag==TRUE) {
+            command=malloc(strlen(GZIP)+strlen(fromfile)+5);
+            strcpy(command, GZIP);
+          }
+          if (bzipflag==TRUE) {
+            command=malloc(strlen(BZIP2)+strlen(fromfile)+5);
+            strcpy(command, BZIP2);
+          }          
+          strcat(command, " -c ");
+          strcat(command, fromfile);
+
+      	  FROMF=popen(command ,"r");
+  	  if (FROMF==NULL) {perror("fopen");return 1;}
+	
+	  RTOF=fopen(tofile,"wb");
+	  if (RTOF==NULL) {perror("fopen");return -1;}
+	  if (write_lock( fileno((FILE*)TOF))==-1) return -1;
+    
+    }
+#endif
   }
 
 
@@ -930,6 +1031,9 @@ if (mode==CBC || mode==CFB) {
    case CFB:
     td = init_mcrypt_cfb(algorithm,keyword,len,IV);
     break;    
+   case OFB:
+    td = init_mcrypt_ofb(algorithm,keyword,len,IV);
+    break;        
    default:
     return 1;
     break;
@@ -1020,6 +1124,40 @@ switch (mode) {
   while(ch==0);
   break;
 
+ case OFB:
+
+  /* Put the IV into the file */
+  if (fwrite(IV, 1, blocksize, TOF) == 0) {perror("fwrite"); return 1;}
+
+  do
+  {
+  /* read 1 byte of the file and store to ciphertext */
+ 
+                   how=fread(ciphertext,1, sizeof(ciphertext),FROMF);
+
+  	           if (how < 1) {
+                    if ( how ==0 && feof(FROMF)==0) {
+                       perror("fread"); 
+                       return 1;
+                    }
+
+                    ch=1;
+                   }
+
+
+
+   /* crc32 */
+    if (bare_flag==FALSE) crc32(ciphertext, how);
+    mcrypt_ofb(td, ciphertext,how);
+         
+    if (ch==0) {
+      if (fwrite(ciphertext, 1, how, TOF) == 0) {perror("fwrite"); return 1;}
+    }
+
+  }
+  while(ch==0);
+  break;
+
 
  case ECB:
   do
@@ -1132,6 +1270,10 @@ switch (mode) {
      end_mcrypt_cfb(td);
      free(IV);
      break;     
+   case OFB:
+     end_mcrypt_ofb(td);
+     free(IV);
+     break;          
   }  
 
     Bzero(keyword, keysize);
@@ -1144,7 +1286,7 @@ switch (mode) {
 
 
 
-
+
 
 /* General Decryption */
 
@@ -1175,8 +1317,7 @@ if (stream_flag==TRUE) {
 	RTOF=stdout;
 }
 else {
-
-	FROMF=fopen(fromfile,"rb");
+  	FROMF=fopen(fromfile,"rb");
 	if (FROMF==NULL) {perror("fopen");return 1;}
         if (read_lock( fileno((FILE*)FROMF))==-1) return 1;
         
@@ -1221,6 +1362,9 @@ else {
     case CFB:
       td = init_mcrypt_ecb( algorithm,keyword,len);
       break;      
+    case OFB:
+      td = init_mcrypt_ecb( algorithm,keyword,len);
+      break;            
     default:
       return 1;
       break;
@@ -1401,6 +1545,43 @@ switch (mode) {
    }
    while(ch==0);
    break;   
+
+ case OFB:
+   end_mcrypt_ecb(td);
+   i=fread( ciphertext, 1, blocksize, FROMF); /* Read the IV */
+   td=init_mcrypt_ofb(algorithm,keyword,len,ciphertext); 
+
+  /* decryption starts here */
+
+   do
+   {
+
+    /* reads 1 byte of the file and stores the to ciphertext */
+
+           i=fread( ciphertext, 1, sizeof(ciphertext), FROMF);
+
+           if (i < 1) {
+             if (ferror(FROMF)!=0) {
+               perror("fread"); 
+               return 1;
+             }
+             ch=1;
+           }
+
+         /* Decrypt the ciphertext */
+
+        mdecrypt_ofb(td, ciphertext, i);
+	
+            if (ch!=1) {
+              if (fwrite(ciphertext,1, i,RTOF)==0) {perror("fwrite"); return 1;}
+                /* CRC32 */
+                if (bare_flag==FALSE) crc32(ciphertext, i);
+            }
+
+   }
+   while(ch==0);
+   break;   
+
  
  default:
    return 1;
@@ -1433,6 +1614,9 @@ switch (mode) {
     case CFB:
       end_mcrypt_cfb(td);
       break;      
+    case OFB:
+      end_mcrypt_ofb(td);
+      break;      
   }
 
   free(ciphertext_old);
@@ -1447,6 +1631,18 @@ switch (mode) {
 
     newcrc32=get_crc32();
     if (newcrc32==fcrc32) {
+#ifdef ZIP     
+    if (stream_flag==FALSE) {
+     if (gzipflag==TRUE) {
+       err_warn(_("Decompressing the output file...\n"));
+       execlp(GZIP, GZIP, "-d", tofile, NULL);
+     }
+     if (bzipflag==TRUE) {
+       err_warn(_("Decompressing the output file...\n"));
+       execlp(BZIP2, BZIP2, "-d", tofile, NULL);
+     }
+    }
+#endif
      return 0; /* crc check passed */
     }
     else {
